package com.brainysoftware.batik;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.batik.dom.svg.SVGDOMImplementation;
import org.apache.batik.transcoder.TranscoderException;
import org.apache.batik.transcoder.TranscoderInput;
import org.apache.batik.transcoder.TranscoderOutput;
import org.apache.batik.transcoder.image.PNGTranscoder;
import org.w3c.dom.CDATASection;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

public class DecorateNameServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;
	
	private final String svgNS = SVGDOMImplementation.SVG_NAMESPACE_URI;
	
	public void doGet(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		// Get the submitted name from the session
		HttpSession session = req.getSession();
		String name = (String) session.getAttribute("name");
		String greeting = "Greetings, " + name + "!";
		
		// Create the SVG document and set its size
		DOMImplementation dom = SVGDOMImplementation.getDOMImplementation();
		Document doc = dom.createDocument(svgNS, "svg", null);
		
		Element root = doc.getDocumentElement();
		root.setAttributeNS(null, "width", "600");
		root.setAttributeNS(null, "height", "120");
		
		// Add an SVG font reference
		ServletContext ctx  = getServletContext();
		File fontFile = new File(ctx.getRealPath("/VinqueSVG.svg"));
		Element defs = doc.createElementNS(svgNS, "defs");
		Element style = doc.createElementNS(svgNS, "style");
		style.setAttributeNS(null, "type", "text/css");
		CDATASection fontRef = doc.createCDATASection(
				"@font-face { font-family: 'Vinque'; src: url(\"" + 
				fontFile.toURL().toString() + 
				"#Font1\") format(svg) }");
		style.appendChild(fontRef);
		defs.appendChild(style);
		
		// Create the filter
		Element filter = doc.createElementNS(svgNS, "filter");
		filter.setAttributeNS(null, "id", "TheFilter");
		
		// Effect 1
		Element blur = doc.createElementNS(svgNS, "feGaussianBlur");
		blur.setAttributeNS(null, "in", "SourceAlpha");
		blur.setAttributeNS(null, "stdDeviation", "4");
		blur.setAttributeNS(null, "result", "blur");
		/*
		// Effect 2
		Element offset = doc.createElementNS(svgNS, "feOffset");
		offset.setAttributeNS(null, "in", "blur");
		offset.setAttributeNS(null, "dx", "4");
		offset.setAttributeNS(null, "dy", "4");
		offset.setAttributeNS(null, "result", "offset");
		*/
		// Effect 3
		Element lighting = doc.createElementNS(svgNS, "feSpecularLighting");
		lighting.setAttributeNS(null, "in", "blur");
		lighting.setAttributeNS(null, "result", "lighting");
		lighting.setAttributeNS(null, "surfaceScale", "5");
		lighting.setAttributeNS(null, "specularConstant", ".75");
		lighting.setAttributeNS(null, "specularExponent", "20");
		lighting.setAttributeNS(null, "lighting-color", "blue");
		Element pointLight = doc.createElementNS(svgNS, "fePointLight");
		pointLight.setAttributeNS(null, "x", "-10000");
		pointLight.setAttributeNS(null, "y", "-10000");
		pointLight.setAttributeNS(null, "z", "10000");
		lighting.appendChild(pointLight);
		
		// Effect 4
		Element composite1 = doc.createElementNS(svgNS, "feComposite");
		composite1.setAttributeNS(null, "in", "lighting");
		composite1.setAttributeNS(null, "in2", "SourceAlpha");
		composite1.setAttributeNS(null, "operator", "in");
		composite1.setAttributeNS(null, "result", "lightWithin");
		/*
		// Effect 5
		Element composite2 = doc.createElementNS(svgNS, "feComposite");
		composite2.setAttributeNS(null, "in", "SourceGraphic");
		composite2.setAttributeNS(null, "in2", "lightWithin");
		composite2.setAttributeNS(null, "operator", "arithmetic");
		composite2.setAttributeNS(null, "k1", "0");
		composite2.setAttributeNS(null, "k2", "1");
		composite2.setAttributeNS(null, "k3", "1");
		composite2.setAttributeNS(null, "k4", "0");
		composite2.setAttributeNS(null, "result", "lighted");
		
		// Effect 6
		Element merge = doc.createElementNS(svgNS, "feMerge");
		Element mergeNode1 = doc.createElementNS(svgNS, "feMergeNode");
		mergeNode1.setAttributeNS(null, "in", "offset");
		Element mergeNode2 = doc.createElementNS(svgNS, "feMergeNode");
		mergeNode2.setAttributeNS(null, "in", "lighted");
		
		merge.appendChild(mergeNode1);
		merge.appendChild(mergeNode2);*/
		
		filter.appendChild(blur);
		//filter.appendChild(offset);
		filter.appendChild(lighting);
		filter.appendChild(composite1);
		//filter.appendChild(composite2);
		//filter.appendChild(merge);
		
		
		defs.appendChild(filter);
		root.appendChild(defs);
		
		// Create the text element
		Element text = doc.createElementNS(svgNS, "text");
		Text greetingText = doc.createTextNode(greeting);
		text.appendChild(greetingText);
		text.setAttributeNS(null, "x", "10");
		text.setAttributeNS(null, "y", "80");
		text.setAttributeNS(null, "font-size", "60");
		text.setAttributeNS(null, "fill", "indigo");
		text.setAttributeNS(null, "font-family", "Vinque");
		text.setAttributeNS(null, "filter", "url(#TheFilter)");
		
		root.appendChild(text);
		
		// Transcode SVG into PNG and send as a response
		resp.setContentType("image/png");
		
		PNGTranscoder t = new PNGTranscoder();
		TranscoderInput input =	new TranscoderInput(doc);		
		TranscoderOutput output = new TranscoderOutput(resp.getOutputStream());
		try {
			t.transcode(input, output);
		}
		catch (TranscoderException te) {
			te.printStackTrace();
		}
    		resp.getOutputStream().close();
	}

	public void doPost(HttpServletRequest req, HttpServletResponse resp) throws IOException {
		doGet(req, resp);
	}
}
